<?php

class RecordBookData
{
    /** @var integer */
    public $id;
    /** @var string */
    public $externalID;
    /** @var integer */
    public $planID;
    /** @var string */
    public $planExternalID;
    /** @var integer */
    public $studentID;
    /** @var string */
    public $studentExternalID;
    /** @var integer */
    public $facultyID;
    /** @var string */
    public $facultyExternalID;
    /** @var string */
    public $status;
    /** @var string */
    public $level;
    /** @var string */
    public $form;
    /** @var string */
    public $speciality;
    /** @var integer */
    public $grade;
    /** @var integer */
    public $group;
}

class StudentData
{
    /** @var integer */
    public $id;
    /** @var string */
    public $externalID;
    /** @var string */
    public $firstName;
    /** @var string */
    public $secondName;
    /** @var string */
    public $lastName;
    /** @var RecordBookData[] */
    public $recordBooks;
}

class Controller_Api_V0_Student extends Controller_Handler_Api
{
    /** Нормализация данных зачетки
     * @param RecordBookData $data
     * @return RecordBookData
     */
    private function normalizeRecordBookData($data)
    {
        if ( !isset($data->id) && isset($data->externalID) ) {
            $ID = Model_RecordBook::withExternalID($data->externalID);
            $data->id = $ID;
        }

        if ( !isset($data->studentID) ) {
            if ( isset($data->studentExternalID) ) {
                $studentID = Model_Student::withExternalID($data->studentExternalID);
                if ($studentID === -1) {
                    $this->badRequestError( $this->makeErrorMsg($data, 'Не найден студент по данному идентификатору.'));
                }

                $data->studentID = $studentID;
            } else {
                $this->badRequestError( $this->makeErrorMsg($data, 'Не указан идентификатор студента.'));
            }
        }

        if ( !isset($data->planID) ) {
            if ( isset($data->planExternalID) ) {
                $planID = Model_Plan::withExternalID($data->planExternalID);

                $data->planID = $planID;
            } else {
                $this->badRequestError( $this->makeErrorMsg($data, 'Не указан идентификатор учебного плана.'));
            }
        }

        if ( !isset($data->facultyID) ) {
            if (  isset($data->facultyExternalID) ) {
                $depid = $this->getFacultyIDByExternalID($data->facultyExternalID);
                if ( $depid === NULL ) {
                    $this->badRequestError( $this->makeErrorMsg($data, 'Ошибка в коде справочника факультета.') );
                }
                $data->facultyID = $depid;
            } else {
                $this->badRequestError( $this->makeErrorMsg($data, 'Не указано подразделение.'));
            }
        }

        switch ($data->status)
        {
            case 'Учится':
                $data->status = 'common';
                break;
            case 'Академ':
                $data->status = 'outlet';
                break;
            case 'Выпустился':
                $data->status = 'expulsion';
                break;
            case 'Отчислен':
                $data->status = 'leave';
                break;
            default:
                Log::instance()->add(Log::ERROR, 'Data contains incorrect record book status: '.$data->status);
        }

        switch ($data->level)
        {
            case 'Академический бакалавр':
            case 'Бакалавр':
                $data->level = 'bachelor';
                break;
            case 'Специалист':
                $data->level = 'specialist';
                break;
            case 'Магистр':
                $data->level = 'master';
                break;
            case 'Аспирант':
                $data->level = 'postgraduate';
                break;
            default:
                Log::instance()->add(Log::ERROR, 'Data contains incorrect record book level: '.$data->level);
        }

        switch ($data->form)
        {
            case 'Очная':
                $data->form = '1';
                break;
            case 'Очно-заочная':
                $data->form = '2';
                break;
            case 'Заочная':
                $data->form = '3';
                break;
            default:
                Log::instance()->add(Log::ERROR, 'Data contains incorrect record book study form: '.$data->form);
        }

        return $data;
    }

    /** Нормализация данных студента
     * @param StudentData $data
     * @return StudentData
     */
    private function normalizeStudentData($data) {
        if ( !isset($data->id) && isset($data->externalID) ) {
            $ID = Model_Student::withExternalID($data->externalID);
            $data->id = $ID;
        }

        echo $data->firstName.' '.$data->secondName.' '.$data->lastName.' $data->id: '.$data->id."\n";
        if ( !isset($data->id) || ($data->id == -1) ) {
            echo "-- Try to find by name !\n";
            $foundedList = Model_Students::searchStudentsByName([$data->firstName, $data->secondName, $data->lastName]);
            if ( count($foundedList) === 1 ) {
                echo '--- Founded by name id: '.$foundedList[0]['ID']."!\n";
                $data->id = $foundedList[0]['ID'];
            }
        }

        foreach ($data->recordBooks as &$recordBook) {
            if ( isset($data->id) ) {
                $recordBook->studentID = $data->id;
            }

            if ( isset($data->externalID) ) {
                $recordBook->studentExternalID = $data->externalID;
            }

            $recordBook = $this->normalizeRecordBookData($recordBook);
        }

        return $data;
    }

    /**
     * @param RecordBookData $data
     * @return integer[]
     */
    private function createRecordBook($data) {
        try {
            $recordbook = Model_RecordBook::make()
                ->studentID($data->studentID)
                ->planID($data->planID)
                ->semesterID(Model_System::$SemesterID)
                ->grade($data->grade)
                ->group($data->group);

            if ( isset($data->externalID) ) {
                $recordbook->externalID($data->externalID);
            }

            $recordbook = $recordbook->create();
        } catch (InvalidArgumentException $e) {
            $this->badRequestError( $this->makeErrorMsg($data, $e->getMessage()) );
        }
        return ['ID' => $recordbook->ID];
    }

    /**
     * @param StudentData $data
     * @return integer[]
     */
    private function createStudent($data) {
        try {
            $student = Model_Student::make()
                ->externalID($data->externalID)
                ->firstName($data->firstName)
                ->secondName($data->secondName)
                ->lastName($data->lastName)
                ->create();

            foreach ($data->recordBooks as $recordBook) {
                $recordBook->studentID = $student->ID;
                $this->createRecordBook($recordBook);
            }
        } catch (InvalidArgumentException $e) {
            $this->badRequestError( $this->makeErrorMsg($data, $e->getMessage()) );
        }
        return ['ID' => $student->ID];
    }

    private function updateRecordBook($data) {
        try {
//            Model_RecordBook::with($data->id)->changeInfo();
        } catch (InvalidArgumentException $e) {
            $this->badRequestError( $this->makeErrorMsg($data, $e->getMessage()) );
        }
    }

    public function action_get_index() {
        $this->response->body( '{ what: get index }' );

        return '{ what: get index }'; //$this->response;
    }

    /**
     * @api {put} api/v0/student/recordBook Add new record book(s)
     * @apiName Add new record book
     * @apiGroup Record books
     * @apiVersion 0.1.2
     * @apiParam {String} token Api key
     * @apiParam {String} [batch] Flag about massive addition
     * @apiParam {String} externalID
     * @apiParam {String} studentExternalID
     * @apiParam {String} planExternalID
     * @apiParam {String} facultyExternalID
     * @apiParam {String} status
     * @apiParam {String} level
     * @apiParam {String} form
     * @apiParam {String} speciality
     * @apiParam {Int} grade
     * @apiParam {Int} group
     */
    public function action_put_recordBook() {
        try {
            if ( $this->request->query('batch') !== NULL ) {
                $data = json_decode($this->request->body());

                $res = [];
                foreach ($data as $item) {
                    $data = $this->normalizeRecordBookData($item);
                    $res[] = $this->createRecordBook($data);
                }
            } else {
                /** @var RecordBookData $data */
                $data = (object)$this->request->query();

                $data = $this->normalizeRecordBookData($data);
                $res[] = $this->createRecordBook($data);
            }
        } catch (Exception $e) {
            $this->badRequestError($e->getMessage());
        }

        return $res;
    }

    /**
     * @api {put} api/v0/student Add new student(s)
     * @apiName Add new student
     * @apiGroup Student
     * @apiVersion 0.1.0
     * @apiParam {String} token Api key
     * @apiParam {String} [batch] Flag about massive addition
     * @apiParam {String} firstName
     * @apiParam {String} secondName
     * @apiParam {String} lastName
     * @apiParam {String} departmentName
     * @apiParam {String} [departmentID]
     * @apiParam {String} jobPositionName
     * @apiParam {String} [jobPositionID]
     * @apiParam {String} status
     */
    public function action_put_index() {
        try {
            if ( $this->request->query('batch') !== NULL ) {
                $data = json_decode($this->request->body());

                $res = [];
                foreach ($data as $item) {
                    $data = $this->normalizeStudentData($item);
                    $res[] = $this->createStudent($data);
                }
            } else {
                $data = (object)$this->request->query();

                $data = $this->normalizeStudentData((object)$data);
                $res[] = $this->createStudent($data);
            }
        } catch (Exception $e) {
            $this->badRequestError($e->getMessage());
        }

        return $res;
    }

    public function action_get_list_by_name() {
        return '{ what: nothing }';
    }
}
